/* @override http://station5:12040/console/styles.css */

.request-id {
	float: right;
}

.button-bar {
	position: fixed;
	top: 5px;
	right: 5px;
}

/* @group request */

.request-line {
	background-color: #b2c3ed;
	padding: 0.5em;
}

.request-method {
	color: #1b009d;
	margin-right: 0.5em;
	text-transform: uppercase;
	font-weight: bold;
}

.request-url {
	margin-right: 0.5em;
}

.request-path {
	font-size: small;
}

.request-content {
	border-style: dotted;
	border-width: 1px;
}

.request-headers {
}

.request-body {
	font-size: small;
	border-top-style: solid;
	border-top-width: 1px;
	white-space: pre;
}


/* @end */

/* @group headers */

dl {
	margin: 0.25em;
}
.headers {
	margin: 0px;
}

.header-key {
	position: absolute;
	width: 25%;
}

.header-value {
	position: relative;
	left: 25%;
}

/* @end */

/* @group response */

.response-line {
	padding: 0.5em;
}

.response-failure {
	background-color: #f00;
}

.response-success {
	background-color: #81ff8c;
}

.response-status {
	color: #1a009d;
	font-weight: bold;
}

.response-content {
	border-style: dotted;
	border-width: 1px;
}

.response-headers {
	
}

.response-body {
	font-size: small;
	border-top-style: solid;
	border-top-width: 1px;
}

/* @end */

/* @group transform */

.transform-line {
	
}

.transform {
	
}

.data {
	
}

/* @end */

/* @group send-data */

.send-data-line {
	
}

.post {
	
}

.target {
	
}

.body {
	
}

/* @end */

/* @group show-hide */

.show-node {
  color: blue;
  cursor: pointer;
  font-size: small;
}

.hide-node {
  color: blue;
  cursor: pointer;
  font-size: small;
}

.body-node {

}

/* @end */