
#shell_input {
    width: 90%;
}

#output {
    padding: 0.5em;
    background-color: #efefef;
}

.log-line {
    background-color: #cdcdcd;
	white-space: pre;
	font-family: "Courier New", Courier, mono;
}

.line-owner {
    display:  none;
    float: right;
}

.input-line {
    background-color: #ababab;
}

.typing-line {
	background-color: #ffffff;
	border: 1px dotted black;
	font-style: italic;
}

.prompt {
    color: blue;
}