/* @override http://station5:12040/test/styles.css */

/* @group outermost nodes */

.button-bar {
	position: fixed;
	top: 5px;
	right: 5px;
}

#test-results {
	
}

#test-timer {
	background-color: #efefef;
}

.test-counter {
	padding: 0.25em;
}

.test-start {
	padding: 0.5em;
	background-color: #efefef;
}

.test-row {
	padding: 0.5em;
}

.test-details {
	border-style: dotted;
	border-width: 1px;
	border-top-style: none;
}

.test-id {
	color: #1a009d;
	font-weight: bold;
}

.test_success {
	background-color: #81ff8c;
}

.test_failure {
	background-color: #f00;
}

.test_error {
	background-color: orange;
}

.test-summary {
	padding: 0.5em;
	background-color: #efefef;
}

/* @end */

/* @group show-hide */

.show-node {
  color: blue;
  cursor: pointer;
  font-size: small;
}

.hide-node {
  color: blue;
  cursor: pointer;
  font-size: small;
}

.body-node {

}

/* @end */

/* @group log nodes */


.log-shell {
	border-top-style: dotted;
	border-top-width: 1px;
}

.log-line {
	
}

/* @end */

/* @group traceback nodes */

.traceback-shell {
	border-top-style: dotted;
	border-top-width: 1px;
}

/* @end */
